@@ -9,6 +9,7 @@
 from aniso8601.builders import TupleBuilder
 from aniso8601.builders.python import PythonTimeBuilder
 from aniso8601.date import parse_date
+from aniso8601.decimalfraction import find_separator, normalize
 from aniso8601.exceptions import ISOFormatError
 from aniso8601.resolution import TimeResolution
 from aniso8601.timezone import parse_timezone
@@ -52,12 +53,9 @@ def get_time_resolution(isotimestr):
         return TimeResolution.Minutes
 
     #Format must be hhmmss, hhmm, or hh
-    if timestr.find('.') == -1:
-        #No time fractions
+    timestrlen = find_separator(timestr)
+    if timestrlen == -1:
         timestrlen = len(timestr)
-    else:
-        #The lowest order element is a fraction
-        timestrlen = len(timestr.split('.')[0])
 
     if timestrlen == 6:
         #hhmmss
@@ -123,7 +121,7 @@ def parse_datetime(isodatetimestr, delimiter='T', builder=PythonTimeBuilder):
     #date and time (<date>T<time>). Fixed offset tzdata will be included
     #if UTC offset is given in the input string.
 
-    isodatestr, isotimestr = isodatetimestr.split(delimiter)
+    isodatestr, isotimestr = isodatetimestr.split(delimiter, 1)
 
     datepart = parse_date(isodatestr, builder=TupleBuilder)
 
@@ -138,7 +136,7 @@ def _parse_hour(timestr, tz, builder):
     if hourstr == '24':
         return builder.build_time(tz=tz)
 
-    return builder.build_time(hh=hourstr, tz=tz)
+    return builder.build_time(hh=normalize(hourstr), tz=tz)
 
 def _parse_minute_time(timestr, tz, builder):
     #Format must be hhmm, hhmm., hh:mm or hh:mm.
@@ -150,7 +148,7 @@ def _parse_minute_time(timestr, tz, builder):
         hourstr = timestr[0:2]
         minutestr = timestr[2:]
 
-    return builder.build_time(hh=hourstr, mm=minutestr, tz=tz)
+    return builder.build_time(hh=normalize(hourstr), mm=normalize(minutestr), tz=tz)
 
 def _parse_second_time(timestr, tz, builder):
     #Format must be hhmmss, hhmmss., hh:mm:ss or hh:mm:ss.
@@ -163,7 +161,8 @@ def _parse_second_time(timestr, tz, builder):
         minutestr = timestr[2:4]
         secondstr = timestr[4:]
 
-    return builder.build_time(hh=hourstr, mm=minutestr, ss=secondstr, tz=tz)
+    return builder.build_time(hh=normalize(hourstr), mm=normalize(minutestr),
+                              ss=normalize(secondstr), tz=tz)
 
 def _split_tz(isotimestr):
     if isotimestr.find('+') != -1:
